classdef BoxConstraints < MinProblem
    % Constraints on a minimization problem taking the form
    %
    %   l <= x <= u
    %
    
    properties
        % Lower bound on each variable
        l
        
        % Upper bound on each variable
        u
    end
    
    methods
        function obj = BoxConstraints(l, u)
            obj.l = l;
            obj.u = u;
        end
        
        function d = project_direction(obj, x, d)
            d(x <= obj.l & d < 0) = 0;
            d(x >= obj.u & d > 0) = 0;
        end
        
        function x = project(obj, x)
            x = min(max(obj.l, x), obj.u);
        end
        
        function A = active_set(obj, x)
            A = x <= obj.l | x >= obj.u;
        end
        
        function B = binding_set(obj, x)
            g = obj.gradient(x);
            B = (x <= obj.l & g > 0) | (x >= obj.u & g < 0);
        end
        
        function beta = breakpoints(obj, x, d)
            % Find breakpoint along each dimension
            beta = repmat(inf, length(obj.l), 1);
            beta(d < 0) = (obj.l(d < 0) - x(d < 0)) ./ d(d < 0);
            beta(d > 0) = (obj.u(d > 0) - x(d > 0)) ./ d(d > 0);
        
            % Sort in increasing order
            beta = sort(beta, 1, 'ascend');
            %beta(beta == inf) = [];
        end
    end
end