classdef MinProblem
    % Abstract class for a minimization problem.
    
    methods (Abstract)
        
        % q = objective(obj, x)
        %
        %   Evaluate objective at point x.
        %
        q = objective(obj, x)
        
        % g = gradient(obj, x)
        %
        %   Find gradient of objective function at point x
        %
        g = gradient(obj, x)
        
        % dq = directional_derivative(obj, x, d)
        %
        %   Find the derivative wrt alpha of
        %
        %     phi(alpha) = q(x + alpha * d)
        %
        %   for q the objective function.
        %
        dq = directional_derivative(obj, x, d)
        
        % ddq = directional_derivative(obj, x, d)
        %
        %   Find the second derivative wrt alpha of
        %
        %     phi(alpha) = q(x + alpha * d)
        %
        %   for q the objective function.
        %
        ddq = directional_derivative2(obj, x, d)
        
        % d = projected_direction(obj, x, d)
        %
        d = project_direction(obj, x, d)
        
        % x = project(obj, x)
        %
        %   Project an arbitrary point into the feasible region.
        %
        x = project(obj, x)
        
        % A = active_set(obj, x)
        %
        %   The indices of active constraints at point x.
        %
        A = active_set(obj, x)
        
        % B = binding_set(obj, x)
        %
        %   The set of constraints which the gradient points into at x
        %
        B = binding_set(obj, x)
        
        % beta = breakpoints(obj, x, d)
        %
        %   The values for which the active set of P[x + \alpha d] changes.
        %
        beta = breakpoints(obj, x, d)
    end
end