classdef QPmv < MinProblem
    % A minimization problem of the form
    %
    %   min   1/2 * x' * Q * x + c' * x
    %
    % Where we can only access Q through matrix-vector products Q * v.
    %
    
    properties
        c
        const
    end
    
    methods (Abstract)
        % Calculate the matrix-vector product Q * v
        prod = mv(obj, v)
    end
    
    methods
        function obj = QPmv(c, const)
            % Create QPmv problem.
            
            if nargin < 2
                const = 0;
            end
            
            obj.c = c;
            obj.const = const;
        end
        
        function q = objective(obj, x)
            q = 1/2 * x' * obj.mv(x) + obj.c' * x + obj.const;
        end
        
        function dq = directional_derivative(obj, x, d)
            g = obj.gradient(x);
            dq = g' * d;
        end
        
        function ddq = directional_derivative2(obj, x, d)
            ddq = d' * obj.mv(d);
        end
            
        function g = gradient(obj, x)
            g = obj.mv(x) + obj.c;
        end
    end
end