function I = boundary_plot(I, seg, seed, S, bound_color)
    % boundary_plot(I, seg, seed, size, bound_color)
    %
    % seed: Optional, pass [] to ignore.
    % bound_color: Optional

    fg_seed_color = [0 1 1];
    bg_seed_color = [1 215/255 0];
    
    if nargin < 5
        bound_color = [1 0 0];
    end
    if nargin < 4
        S = [size(I, 1) size(I, 2)];
    else
        I = reshape(I, S);
    end
    if nargin < 3
        seed = [];
    end
    
    % Make I an RGB image
    %I = (I - min(I(:))) / (max(I(:)) - min(I(:)));
    if ndims(I) == 2
        I = repmat(I, [1 1 3]);
    end
    
    % Move parameter colors to same range as image
    if max(I(:)) > 1
        bound_color = bound_color * 255;
        fg_seed_color = fg_seed_color * 255;
        bg_seed_color = bg_seed_color * 255;
    end
    
    m = S(1);
    n = S(2);
    
    function I = apply_color(I, mask, color)
        for i=1:3
            select = zeros([m n 3]);
            select(:, :, i) = mask;
            
            I(select ~= 0) = color(i);
        end
    end
    
    % Find boundaries
    seg = reshape(seg, S);
    P = bwperim(seg > 1/2);
    
    % Highlight boundaries
    I = apply_color(I, P, bound_color);
    
    % Highlight seed points
    if numel(seed) ~= 0
        I = apply_color(I, seed > 0, fg_seed_color);
        I = apply_color(I, seed < 0, bg_seed_color);
    end
    
    % Plot the image itself
    image(I);
    
    % Remove axis labels
    set(gca, 'XTick', [], 'YTick', []);
end