function [H C] = cluster_hist(features, masks, k, max_dist)
    
    if nargin < 3
        k = -1;
        %k = 64;
    end
    if nargin < 4
        max_dist = 1e2;
    end

    % Handle sizing
    d = size(features{1}, ndims(features{1}));
    nimages = numel(features);
    
    start = zeros(nimages, 1);
    stop = zeros(nimages, 1);
    npixels = zeros(nimages, 1);
    
    start(1) = 1;
    for i=1:nimages
        npixels(i) = sum(masks{i}(:));
        stop(i) = start(i) + npixels(i) - 1;

        if i == nimages
            n = stop(i);
        else
            start(i+1) = stop(i) + 1;
        end
    end
    
    % Concatenate into one big dataset
    f = zeros(d, n);
    for i=1:nimages
        fi = reshape(features{i}, numel(features{i})/d, d)';
        f(:, start(i):stop(i)) = fi(:, masks{i}(:) ~= 0);
    end
    
    % Perform spectral clustering
    fprintf('Clustering...\n');
    %C = spectral_cluster(f, k);
    tic
    %C = kmeans(f', k, 'Start', 'cluster', 'Replicates', 5, 'EmptyAction',
    
    
    if k ~= -1
        % Run k-means with the given # of clusters
    fprintf(1, 'kmeans...\n');
        [C idx] = vl_kmeans(f,  k, 'distance', 'l1', 'algorithm', 'elkan');
    else
        % Initially estimate number of clusters with mean-shift
        %sigma = estimate_bandwidth(f);
        %sigma = 60;
        %fprintf('global sigma = %g\n', sigma);
        
        % Cluster with Gonzalez85
        [idx C] = gonzalez85(f, size(f, 2), 60);
        idx = idx + 1;
        k = size(C, 2)
    end
    
    toc
    
    % Reshape cluster assignments into histograms
    H = cell(size(features));
    for i=1:nimages
        H{i} = zeros(size(masks{i}));
        H{i}(masks{i} ~= 0) = idx(start(i):stop(i));
    end
end