function [flat sizes] = flatten_image(I)
    % [flat sizes] = flatten_image(I)
    %
    %   Reduce an image or set of images to flat vectors.
    %
    %     I: A 2D image or a cell array of 2D images.
    %  flat: I, except as a single column vector.
    % sizes: The original size(s) of I.  If I was a cell array, sizes(i, :)
    %        was the size of I{i}.
    
    if iscell(I)
        % Handle cell array of multiple images
        flat = cell(size(I));
        sizes = zeros(numel(size(I)), 2);
        
        for i=1:numel(I)
            [flat{i}, sizes(i, :)] = flatten_image(I{i});
        end
    else
        % Actual flattening operation
        sizes = size(I);
        flat = reshape(I, numel(I), 1);
    end
end