function H = histogram_assignment_matrix(m, hI)
    % H = histogram_assignment_matrix(m, hI)
    %
    %   Create matrix which assigns segmentation labels to additions
    %   to the corresponding histogram bin.  Thus, given a binary vector
    %   x which selects a subset of the image, the product H*x will be
    %   the histogram of that subset.
    %
    %  m: The total number of histogram bins.
    % hI: The image as a column vector, mapped to histogram bins.  May be
    %     a cell array where each element is such a vector.
    %
    %  H: The histogram assigment matrix.  If 'hI' was a cell array, 'H'
    %     will be a cell array of corresponding elements.
    %
    
    if iscell(hI)
        H = cell(size(hI));
        
        for i=1:numel(hI)
            H{i} = histogram_assignment_matrix(m, hI{i});
        end
    else
        n = length(hI);
        
        col = 1:n;
        col = col(hI > 0);
        hI = hI(hI > 0);
        
        H = sparse(hI, col, ones(length(col),1), m, n);
    end
end