import os, os.path, subprocess
import re

from SCons.Script import *

mexre = re.compile('.*mex\s*=\s*(.*)')

def mexext(matlab_bin):
    """Get extension for mex files."""

    # Run Matlab
    cmd = [matlab_bin, '-nodisplay', '-nojvm']
    devnull = file(os.devnull)

    proc = subprocess.Popen(cmd, stdout = subprocess.PIPE, stdin = subprocess.PIPE, stderr = devnull)

    # Issue mexext command
    proc.stdin.write("fprintf(1, 'mex = %s\\n', mexext)\nexit\n")

    # Parse output
    for line in proc.stdout:
        m = mexre.match(line)
        if m:
            return m.groups()[0]


def mexGenerator(source, target, env, for_signature):
    return '$MEX $MEXFLAGS $_CPPDEFFLAGS $_CPPINCFLAGS -o $TARGET $SOURCES $_LIBDIRFLAGS $_LIBFLAGS'

def matlabRoot(env):
    if os.path.isabs(env['MATLAB']):
        matlab_path = env['MATLAB']
    else:
        cmd = ['which', env['MATLAB']]
        proc = subprocess.Popen(cmd, stdout = subprocess.PIPE)
        matlab_path = proc.stdout.read()

    return os.path.dirname(os.path.dirname(matlab_path))

def generate(env):
    env.SetDefault(MEX = 'mex')
    env.SetDefault(MATLAB = 'matlab')

    if not env.Detect(env['MATLAB']):
        del env['MEX']
        del env['MATLAB']
        return

    env.SetDefault(MEXSUFFIX = '.' + mexext(env['MATLAB']))
    env.SetDefault(MATLABROOT = matlabRoot(env))
    env.SetDefault(MEXINCPATH = '$MATLABROOT/extern/include')

    if env['MEXSUFFIX'] in ['.mexa64', '.mexmaci64', '.mexw64']:
        env.PrependUnique(MEXFLAGS = ['-largeArrayDims'])

    env.AppendUnique(CPPPATH = ['$MEXINCPATH'])

    #print env.Dump()
    #Exit(0)


    # Create builder
    bld = env.Builder(generator = mexGenerator,
                      suffix = env['MEXSUFFIX'],
                      source_scanner = CScanner)
    env.Append(BUILDERS = {'Mex': bld})

def exists(env):
    return True
