function [r mask] = winn_filters(I)
    % responses = winn_filters(I)
    %
    %   Runs the filter bank from Winn et al (ICCV 2005) on RGB image I.
    %
    
    if iscell(I)
        r = cell(size(I));
        mask = cell(size(I));
        
        for i=1:numel(I)
            [r{i} mask{i}] = winn_filters(I{i});
        end
        
        return;
    end
    
    if isa(I, 'uint8')
        I = im2double(I);
    end
    
    % Convert to correct color space
    %[L, a, b] = rgb2lab(I);
    %Lab = cat(3, L, a, b);
    Ctrans = makecform('srgb2lab');
    Lab = applycform(I, Ctrans);
    L = Lab(:, :, 1);
    
    r = zeros([size(I,1) size(I, 2) 17]);
    mask = ones(size(I, 1), size(I, 2));
    
    % Gaussian filters
    for i=1:3
        for j=1:3
            sigma = 2^(j-1);
            
            K = gaussian_kernel(sigma);
            r(:, :, 3*(i-1) + j) = conv2(Lab(:, :, i), K, 'same');
            mask = mask & conv2_valid(Lab(:, :, i), K);
        end
    end
    
    % Laplacian-of-gaussian filters
    for i=1:4
        sigma = 2^(i-1);
        K = LoG_kernel(sigma);
        r(:, :, 9+i) = conv2(L, K, 'same');
        mask = mask & conv2_valid(L, K);
    end
    
    % Derivative-of-gaussian filters
    for i=1:2
        sigma = 2^i;
        
        K = gaussian_derivative_kernel(sigma, [1, 0]);
        r(:, :, 12 + 2*i) = conv2(L, K, 'same');
        mask = mask & conv2_valid(L, K);
        
        K = gaussian_derivative_kernel(sigma, [0, 1]);
        r(:, :, 13 + 2*i) = conv2(L, K, 'same');
        mask = mask & conv2_valid(L, K);
    end
    
    % Zero entries outside the mask
    %r(repmat(mask, [1 1 17]) == 0) = 0;
end